local EnteringWorldOrVariablesLoaded = 0
local UpdateMacroCount = 0
local EventFrame
local EventHandlersTable = {}
local toolsFrame, overlayFrame, minimapButton
local IsInit

local Containers = {}
local createBarMode = false

--Button Facade stuff
local LBF
local BF_Table = {}
TEB_LBFMasterGroup = nil
local BF_SETTINGS_GROUP = "Button Facade"

local function ButtonFacadeCallback(self, SkinID, Gloss, Backdrop, Group, Button, Colors)
	-- If no group is specified, save the data as the root add-on skin.
	-- This will allow the ButtonFacade GUI to display it correctly.
	if not(Group) then
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, "SkinID"}, SkinID)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, "Gloss"}, Gloss)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, "Backdrop"}, Backdrop)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, "Colors"}, Colors)
	else
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, Group, "SkinID"}, SkinID)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, Group, "Gloss"}, Gloss)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, Group, "Backdrop"}, Backdrop)
		TinyExtraBarsG:Set({BF_SETTINGS_GROUP, Group, "Colors"}, Colors)
	end
end

TEB_KeybindMode = nil
TEB_SettingsMode = true

function TinyExtraBars_RemoveContainer(id)
	if InCombatLockdown() then
		return
	end
	
	Containers[id]:Hide()
	local size = #Containers
	for i = id, size - 1 do
		Containers[i] = Containers[i + 1]
		Containers[i]:SetID(i)
		TinyExtraBarsPC:Set({'Containers', i}, TinyExtraBarsPC:Get({'Containers', i + 1}, nil))
	end
	table.remove(Containers, size)
	TinyExtraBarsPC:Remove({'Containers'}, size)
end

function TinyExtraBars_RegisterContainer(id, container)
	Containers[id] = container
end

-- Tools Frame

function TinyExtraBarsToolsFrame_OnShow()
	if InCombatLockdown() then
		return
	end
	
	TEB_SettingsMode = true
	TinyExtraBarsPC:Set({'ToolsFrame', 'SettingsMode'}, TEB_SettingsMode)
	for k, v in ipairs(Containers) do
		for kf, vf in ipairs(v.FrameList) do
			vf:DisableVisibilityDriver()
		end
		v:Show()
		v:SetEmptyButtonsVisible(true)
		PanelTemplates_SetTab(v, v.activeTab)
		TinyExtraBarsTabButton_OnClick(v.tabs[v.activeTab])
	end
end

function TinyExtraBarsToolsFrame_OnHide()
	if InCombatLockdown() then
		return
	end
	
	CloseDropDownMenus(1)
	TEB_SettingsMode = false
	TinyExtraBarsPC:Set({'ToolsFrame', 'SettingsMode'}, TEB_SettingsMode)
	for k, v in ipairs(Containers) do
		v:Hide()
		v:SetEmptyButtonsVisible(false)
		for kf, vf in ipairs(v.FrameList) do
			vf:UpdateVisibilityDriver()
		end
	end
end

function TinyExtraBarsToolsFrame_OnMouseDown(self, button)
	if button == "LeftButton" then
		self:StartMoving()
	end
end

function TinyExtraBarsToolsFrame_OnMouseUp(self)
	self:StopMovingOrSizing()
	TinyExtraBarsPC:Set({'ToolsFrame', 'pos', 'left'}, self:GetLeft())
	TinyExtraBarsPC:Set({'ToolsFrame', 'pos', 'top'}, self:GetTop())
end

function TinyExtraBarsToolsFrame_SetButtonSize(value)
	if InCombatLockdown() then
		return
	end
	
	TinyExtraBarsG:Set({'ButtonSize'}, value)
	TEB_BUTTON_SIZE = value
	--TEB_BUTTON_SCALE = TEB_BUTTON_SIZE / TEB_DEFAULT_BUTTON_SIZE
	--print("scale", TEB_BUTTON_SCALE)
	for _, v in ipairs(Containers) do
		local rows = v.rows
		local cols = v.cols
		v:SetSize(rows, cols)
		for _, vf in ipairs(v.FrameList) do
			vf:SetSize(rows, cols)
			for r, _ in ipairs(vf.ButtonList) do
				for c, btn in ipairs(vf.ButtonList[r]) do
					btn:SetSize(value, value)
					--btn:SetScale(TEB_BUTTON_SCALE)
					btn:SetAnchor(r, c)
				end
			end
		end
	end
end

function TinyExtraBarsToolsFrame_SetButtonSpace(value)
	if InCombatLockdown() then
		return
	end
	
	TinyExtraBarsG:Set({'ButtonSpace'}, value)
	TEB_BUTTON_SPACING = value
	for k, v in ipairs(Containers) do
		local rows = v.rows
		local cols = v.cols
		v:SetSize(rows, cols)
		for _, vf in ipairs(v.FrameList) do
			vf:SetSize(rows, cols)
			for r, _ in ipairs(vf.ButtonList) do
				for c, btn in ipairs(vf.ButtonList[r]) do
					btn:SetAnchor(r, c)
				end
			end
		end
	end
end

function TinyExtraBars_UnbindButtons(key)
	for k, v in ipairs(Containers) do
		local rows = v.maxRows
		local cols = v.maxCols
		for _, vf in ipairs(v.FrameList) do
			for r, _ in ipairs(vf.ButtonList) do
				for c, btn in ipairs(vf.ButtonList[r]) do
					if btn.keybind == key then
						btn:SetKey(nil)
					end
				end
			end
		end
	end
end

local function Init()
	if InCombatLockdown() then
		print("TinyExtraBars will be loaded on leaving combat")
		EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	assert(LibStub, "LibStub not found")

	-- EasyStorage
	local storage = LibStub("EasyStorage-1.2", false)
	
	TinyExtraBars_Global = TinyExtraBars_Global or {}
	TinyExtraBarsG = storage:New(TinyExtraBars_Global)
	TinyExtraBars_PerChar = TinyExtraBars_PerChar or {}
	TinyExtraBarsPC = storage:New(TinyExtraBars_PerChar)

	
	TEB_BUTTON_SIZE = TinyExtraBarsG:Get({'ButtonSize'}, TEB_DEFAULT_BUTTON_SIZE)	
	TEB_BUTTON_SPACING = TinyExtraBarsG:Get({'ButtonSpace'}, 4)
	--TEB_BUTTON_SCALE = TEB_BUTTON_SIZE / TEB_DEFAULT_BUTTON_SIZE

	--Button Facade
	LBF = LibStub("LibButtonFacade", true)
	if (LBF) then
		TEB_LBFMasterGroup = LBF:Group("TinyExtraBars")
		LBF:RegisterSkinCallback("TinyExtraBars", ButtonFacadeCallback, BF_Table)
		TEB_LBFMasterGroup:Skin(
			TinyExtraBarsG:Get({BF_SETTINGS_GROUP, "SkinID"}, nil),
			TinyExtraBarsG:Get({BF_SETTINGS_GROUP, "Gloss"}, nil),
			TinyExtraBarsG:Get({BF_SETTINGS_GROUP, "Backdrop"}, nil),
			TinyExtraBarsG:Get({BF_SETTINGS_GROUP, "Colors"}, nil))
	end

	-- LibKeyBound
	TEB_LibKeyBound = LibStub("LibKeyBound-1.0", false)
	if (TEB_LibKeyBound) then
		TEB_LibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_ENABLED")
		TEB_LibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_DISABLED")
		TEB_LibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_MODE_COLOR_CHANGED")
	end

	-- toolsFrame
	toolsFrame = CreateFrame('Frame', 'TinyExtraBarsToolsFrame', UIParent, 'TinyExtraBarsToolsFrameTemplate')
	toolsFrame:ClearAllPoints()
	local left = TinyExtraBarsPC:Get({'ToolsFrame', 'pos', 'left'}, 935)
	local top = TinyExtraBarsPC:Get({'ToolsFrame', 'pos', 'top'}, 600)
	toolsFrame:ClearAllPoints()
	toolsFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top)
	TEB_SettingsMode = TinyExtraBarsPC:Get({'ToolsFrame', 'SettingsMode'}, true)
	toolsFrame.sliderButtonSize:SetValue(TEB_BUTTON_SIZE)
	_G[toolsFrame.sliderButtonSize:GetName().."Text"]:SetText(TEB_BUTTON_SIZE)
	toolsFrame.sliderButtonSpace:SetValue(TEB_BUTTON_SPACING)
	_G[toolsFrame.sliderButtonSpace:GetName().."Text"]:SetText(TEB_BUTTON_SPACING)

	RegisterStateDriver(toolsFrame, "visibility", "[combat] hide; [vehicleui] hide")

	-- overlayFrame
	overlayFrame = CreateFrame('Frame', 'TinyExtraBarsCreateBarOverlay', UIParent, 'TinyExtraBarsCreateBarOverlayTemplate')

	-- minimap button
	minimapButton = CreateFrame('Button', 'TinyExtraBarsMinimapButton', UIParent, 'TinyExtraBarsMinimapButtonTemplate')

	-- create containers
	local cont = TinyExtraBarsPC:Get({'Containers'}, {})
	for k, v in ipairs(cont) do
		--print(k, v)
		if v and v.tabs then
			hiContainerId = k
			local left = TinyExtraBarsPC:Get({'Containers', k, 'pos', 'left'})
			local top = TinyExtraBarsPC:Get({'Containers', k, 'pos', 'top'})
			local cols = TinyExtraBarsPC:Get({'Containers', k, 'cols'}, 6)
			local rows = TinyExtraBarsPC:Get({'Containers', k, 'rows'}, 2)
			local cf = TEB_Container_New(k, left, top, rows, cols)
			Containers[k] = cf		
		
			if toolsFrame:IsShown() then
				cf:Show()
			end
		else
			TinyExtraBarsPC:Set({'Containers', k}, nil)
		end
	end
	
	IsInit = true
	
	if TEB_SettingsMode then
		toolsFrame:Show()
	end
end

function TinyExtraBarsMinimapButton_OnClick(self, button)
	if IsInit then
		if (button == "LeftButton") then
			if toolsFrame:IsShown() then
				toolsFrame:Hide()
			elseif not(InCombatLockdown()) then
				toolsFrame:Show()
			end
		end
	end
end

function TinyExtraBarsMinimapButton_OnLoad(self)
	local x = TinyExtraBarsG:Get({'minimap_button', 'x'}, 62 - (80 * cos(5)))
	local y = TinyExtraBarsG:Get({'minimap_button', 'y'}, (80 * sin(5)) - 62)
	self:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", x, y)
end

function TinyExtraBarsMinimapButton_OnMouseUp(self, button)
	if (button == "RightButton") then
		local minimap = _G["Minimap"]
		if minimap then
			local xf = self:GetLeft()
			local yf = self:GetTop()
			local xm = minimap:GetLeft()
			local ym = minimap:GetTop()
			--local s = frame:GetEffectiveScale()
			--x, y = x/s, y/s
			local x = xf - xm
			local y = yf - ym
			TinyExtraBarsG:Set({'minimap_button', 'x'}, x)
			TinyExtraBarsG:Set({'minimap_button', 'y'}, y)
		end
	end
end

function TinyExtraBarsToggleCreateBar(forcedOff)
	if forcedOff then
		createBarMode = false
	else
		createBarMode = not(createBarMode)
	end
	
	if InCombatLockdown() then
		createBarMode = false
		return
	end
	
	if createBarMode then
		SetCursor("REPAIRNPC_CURSOR")
		overlayFrame:Show()
	else
		SetCursor(nil)
		overlayFrame:Hide()
	end
end

local function OnEvent(self, event, ...)
	if EventHandlersTable[event] then
		EventHandlersTable[event](self, ...)
	elseif event == "PLAYER_ENTERING_WORLD" or event == "VARIABLES_LOADED" or event == "UPDATE_MACROS" or event == "PET_JOURNAL_LIST_UPDATE" then
		EnteringWorldOrVariablesLoaded = EnteringWorldOrVariablesLoaded + 1
		if event == "PLAYER_ENTERING_WORLD" then EventFrame:UnregisterEvent("PLAYER_ENTERING_WORLD") end
		if event == "VARIABLES_LOADED" then EventFrame:UnregisterEvent("VARIABLES_LOADED") end
		if event == "PET_JOURNAL_LIST_UPDATE" then EventFrame:UnregisterEvent("PET_JOURNAL_LIST_UPDATE") end
		if event == "UPDATE_MACROS" then
			--local numglobal, numperchar = GetNumMacros()
			--print('UpdateMacroCount = '..UpdateMacroCount..', numglobal = '..numglobal..', numperchar = '..numperchar)
			UpdateMacroCount = UpdateMacroCount + 1
			if UpdateMacroCount > 1 then
				EventFrame:UnregisterEvent("UPDATE_MACROS")
			end
		end
		if EnteringWorldOrVariablesLoaded > 4 then
			Init()
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		EventFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")
		Init()
	elseif event == "PLAYER_REGEN_DISABLED" then
		TEB_SettingsMode = false
	end
end

EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:RegisterEvent("VARIABLES_LOADED")
EventFrame:RegisterEvent("UPDATE_MACROS")
EventFrame:RegisterEvent("PET_JOURNAL_LIST_UPDATE")
EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")

EventFrame:SetScript("OnEvent", OnEvent)

function EventFrame:LIBKEYBOUND_ENABLED()
	TEB_KeybindMode = true
end

function EventFrame:LIBKEYBOUND_DISABLED()
	TEB_KeybindMode = nil
end

function EventFrame:LIBKEYBOUND_MODE_COLOR_CHANGED()
	print("color changed")
end

